using System;
using ExchangeNetwork;

namespace CDX_REQUESTOR_CLIENT
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Client
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
//		[STAThread]
		static void Main(string[] args)
		{
						
			string URL = "https://dev.epacdxnode.net/cdx/services/NetworkNodePortType_V10";
		    string backendURL = "https://dev.epacdxnode.net/cdx/services/DataflowDocumentService";
			
			string USERNAME = "cdx";
			string PASSWORD = "test";
			string DATAFLOWTYPE = "FRS";
			string DOCUMENTNAME = "testSubmit.xml";
			string DOCUMENTTYPE = "XML";
			string SERVICEREQUEST ="GetFacilityByName";
			string [] SERVICEPARAMETERVALUES = {"NE", "BECHIUS FARM"};
			string SERVICETYPE = "Query";

			try
			{
				NodeRequestor requestor = new NodeRequestor(URL);

				//please note we are going over https
				
				Console.Out.WriteLine("[client]: Using URL {0}", URL);

				//first lets ping
				string response = requestor.NodePing("hello");
				Console.Out.WriteLine("[NodePing]: Recieved response {0}", response );
				String token = requestor.Authenticate(USERNAME, PASSWORD);
				Console.Out.WriteLine("[Authenticate]: Recieved security token {0}", token );
				
				byte[] byteArray = {111, 111};
				NodeDocument document = new NodeDocument();
				document.name=DOCUMENTNAME;
				document.type=DOCUMENTTYPE;
				document.content=byteArray;
				NodeDocument [] documentArray = new NodeDocument [1];
				documentArray[0] = document;
				
				//String transactionId= "test";
				String transactionId =
					requestor.Submit(token, null, DATAFLOWTYPE, documentArray);
				Console.Out.WriteLine("[Submit]: Recieved transactionId from submit {0}", transactionId );

				//check status of submited document
				String status =
					requestor.GetStatus(token, transactionId);
				Console.Out.WriteLine("[GetStatus]: Status of transaction {0} is {1}", transactionId, status );

				//download the document that was just sumitted
				requestor.Download(token, transactionId, DATAFLOWTYPE, ref documentArray);

				Console.Out.WriteLine("[Download]: Recieved {0} documents. ",documentArray.Length);

				for (int i=0; i<documentArray.Length; i++)
				{

					Console.Out.WriteLine("[Download]: Document {0}", i);
					Console.Out.WriteLine("[Download]: name={0}. ", documentArray[i].name);
					Console.Out.WriteLine("[Download]: type={0}. ", documentArray[i].type);
					Console.Out.WriteLine("[Download]: content Length in bytes ={0}. ", documentArray[i].content.Length);

				}

				//lets perform query agains FRS
				string queryResultContent =
					requestor.Query(token, SERVICEREQUEST, 0, -1, SERVICEPARAMETERVALUES);
				Console.Out.WriteLine("[Query]: Begin:" );
				
				Console.Out.WriteLine(queryResultContent);
				Console.Out.WriteLine("[Query]: End");

 				//notify
 				string notifytransaction =
 					requestor.Notify(token, null, DATAFLOWTYPE, documentArray);
 				Console.Out.WriteLine("[Notify]: recieved transactionid from notify {0}", notifytransaction );

				//perform solicit
				String solicitTransactionId =
					requestor.Solicit(token, null, SERVICEREQUEST,  SERVICEPARAMETERVALUES);
				Console.Out.WriteLine("[Solicit]: Recieved transactionId from solicit {0}",solicitTransactionId );

				//get list services
				String [] services =
					requestor.GetServices(token, SERVICETYPE);
				Console.Out.WriteLine("[GetServices]: List of supported services: ");
				for (int counter = 0; counter < services.Length; counter++) 
				{
					String service = services[counter];
					Console.Out.WriteLine(counter + " " + service);
				}
				Console.Out.WriteLine("[GetServices]: End of List of supported services.");

				// Peform document status notification on Node back-end
				System.Text.ASCIIEncoding  encoding=new System.Text.ASCIIEncoding();
				byte[] byteNotifyArray = encoding.GetBytes("DOCUMENT PROCESSING FAILED TO INVALID FACILITY ID");
				NodeDocument notifyDocument = new NodeDocument();
				notifyDocument.name="02f5bffd-3448-4098-9da7-7570e37f9ff9";  //overload name with doc id
				notifyDocument.type="FAILED";  // overload type with document status
				notifyDocument.content=byteNotifyArray;
				NodeDocument [] documentNotifyArray = new NodeDocument [1];
				documentNotifyArray[0] = notifyDocument;

				NodeRequestor backendRequestor = new NodeRequestor(backendURL);
				notifytransaction =
					backendRequestor.Notify(token, null, "http://www.exchangenetwork.net/node/status", documentNotifyArray);
				Console.Out.WriteLine("[Notify]: recieved transactionid from backend notify {0}", notifytransaction );


			}
			catch (Exception ex)
			{
				Console.Out.WriteLine (ex.ToString());
			}


		}
	}
}
